/*
 * Copyright (C) 2010 Atmark Techno, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/platform_device.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>
#include <linux/mtd/cfi.h>
#include <linux/mtd/armadillo.h>
#include <linux/gpio.h>
#include <linux/gpio_keys.h>
#include <linux/i2c.h>
#include <linux/input.h>
#include <linux/leds.h>
#include <linux/pwm_backlight.h>
#include <linux/regulator/fixed.h>
#include <linux/regulator/machine.h>
#include <linux/mx25_fec.h>
#include <linux/fsl_devices.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/time.h>
#include <asm-generic/gpio.h>
#include <mach/common.h>
#include <mach/hardware.h>
#include <mach/mx25.h>
#include <mach/iomux.h>
#include <mach/imx-uart.h>
#include <mach/i2c.h>
#include <mach/mmc.h>
#include <mach/armadillo440_wm8978.h>

#include "mx25_pins.h"
#include "armadillo400_gpio.h"
#include "devices.h"
#include "watchdog.h"

struct mxc_ext_gpio {
	const char *name;
	u32 gpio;
	int default_mode;
};

static struct mxc_ext_gpio armadillo400_gpio_list[] = {
#if defined(CONFIG_ARMADILLO400_CON9_21_GPIO1_8)
	{"CON9_21",  GPIO(1, 8),  GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_22_GPIO1_9)
	{"CON9_22",  GPIO(1, 9),  GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_23_GPIO1_10)
	{"CON9_23",  GPIO(1, 10), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_24_GPIO1_11)
	{"CON9_24",  GPIO(1, 11), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_25_GPIO1_16)
	{"CON9_25",  GPIO(1, 16), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_26_GPIO)
	{"CON9_26",  GPIO(2, 22), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_27_GPIO2_21)
	{"CON9_27",  GPIO(2, 21), GPIOF_OUT_INIT_LOW},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_28_GPIO3_15)
	{"CON9_28",  GPIO(3, 15), GPIOF_OUT_INIT_LOW},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_11_GPIO1_17)
	{"CON9_11",  GPIO(1, 17), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_12_GPIO1_29)
	{"CON9_12",  GPIO(1, 29), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_13_GPIO1_18)
	{"CON9_13",  GPIO(1, 18), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_14_GPIO1_30)
	{"CON9_14",  GPIO(1, 30), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_15_GPIO1_7)
	{"CON9_15",  GPIO(1, 7),  GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_16_GPIO1_31)
	{"CON9_16",  GPIO(1, 31), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_17_GPIO4_21)
	{"CON9_17",  GPIO(4, 21), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_18_GPIO1_6)
	{"CON9_18",  GPIO(1, 6),  GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_1_GPIO3_17)
	{"CON9_1",   GPIO(3, 17), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_2_GPIO3_14)
	{"CON9_2",   GPIO(3, 14), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON11_42_GPIO_2_31)
	{"CON11_42", GPIO(2, 31), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON11_43_GPIO_3_0)
	{"CON11_43", GPIO(3, 0),  GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON11_44_GPIO_3_1)
	{"CON11_44", GPIO(3, 1),  GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON11_45_GPIO_3_2)
	{"CON11_45", GPIO(3, 2),  GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON11_46_GPIO_3_3)
	{"CON11_46", GPIO(3, 3),  GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON11_47_GPIO_3_4)
	{"CON11_47", GPIO(3, 4),  GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON11_48_GPIO1_0)
	{"CON11_48", GPIO(1, 0),  GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON11_49_GPIO1_1)
	{"CON11_49", GPIO(1, 1),  GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON14_3_GPIO1_2)
	{"CON14_3",   GPIO(1, 2), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON14_4_GPIO1_3)
	{"CON14_4",   GPIO(1, 3), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_3_GPIO1_14)
	{"CON9_3",   GPIO(1, 14), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_4_GPIO1_27)
	{"CON9_4",   GPIO(1, 27), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_5_GPIO1_15)
	{"CON9_5",   GPIO(1, 15), GPIOF_IN},
#endif
#if defined(CONFIG_ARMADILLO400_CON9_6_GPIO1_28)
	{"CON9_6",   GPIO(1, 28), GPIOF_IN},
#endif
};

extern void gpio_common_active(void);
static void __init armadillo400_init_gpio(void)
{
	struct mxc_ext_gpio *gpios = armadillo400_gpio_list;
	int i;

	gpio_common_active();

	for (i = 0; i < ARRAY_SIZE(armadillo400_gpio_list); i++) {
		gpio_request(gpios[i].gpio, gpios[i].name);
		gpio_export(gpios[i].gpio, true);
		if (gpios[i].default_mode == GPIOF_IN) {
			gpio_direction_input(gpios[i].gpio);
		} else if (gpios[i].default_mode == GPIOF_OUT_INIT_LOW) {
			gpio_direction_output(gpios[i].gpio, 0);
		} else {
			gpio_direction_output(gpios[i].gpio, 1);
		}
	}
}

static int armadillo440_wm8978_audio_init(struct platform_device *pdev)
{
	gpio_activate_audio_ports();
	return 0;
}

static struct armadillo440_wm8978_audio_data armadillo440_wm8978_audio_data = {
	.init = armadillo440_wm8978_audio_init,
};

static struct platform_device armadillo440_wm8978_audio_device = {
	.name = "armadillo440-wm8978-audio",
	.id = 0,
};

static void __init armadillo440_init_audio(void)
{
	mxc_register_device(&armadillo440_wm8978_audio_device,
			    &armadillo440_wm8978_audio_data);
}

#define GPIO_LED(_name, _default_trigger, _gpio, _active_low)	\
{								\
	.name = _name,						\
	.default_trigger = _default_trigger,			\
	.gpio = _gpio,						\
	.active_low = _active_low,				\
}

static struct gpio_led armadillo400_led_pins[] = {
	GPIO_LED("red",    "default-on", GPIO(3, 28), 0),
	GPIO_LED("green",  "default-on", GPIO(3, 29), 0),
	GPIO_LED("yellow", "none",       GPIO(4, 30), 0),
};

static struct gpio_led_platform_data armadillo400_gpio_led_data = {
	.leds = armadillo400_led_pins,
	.num_leds = ARRAY_SIZE(armadillo400_led_pins),
};

static struct platform_device armadillo400_gpio_led_device = {
	.name = "leds-gpio",
	.id = 0,
};

static void __init armadillo400_init_led(void)
{
	mxc_register_device(&armadillo400_gpio_led_device,
			    &armadillo400_gpio_led_data);
}

static struct gpio_keys_button armadillo400_key_buttons[] = {
	{KEY_ENTER, GPIO(3, 30), 1, "SW1",     EV_KEY, 1},
#if defined(CONFIG_MACH_ARMADILLO440)
	{KEY_BACK,  GPIO(2, 20), 1, "LCD_SW1", EV_KEY, 1},
#endif
#if defined(CONFIG_ARMADILLO400_CON11_40_GPIO_2_29)
	{KEY_MENU,  GPIO(2, 29), 1, "LCD_SW2", EV_KEY, 1},
#endif
#if defined(CONFIG_ARMADILLO400_CON11_41_GPIO_2_30)
	{KEY_HOME,  GPIO(2, 30), 1, "LCD_SW3", EV_KEY, 1},
#endif
};

static struct gpio_keys_platform_data armadillo400_gpio_key_data = {
	.buttons = armadillo400_key_buttons,
	.nbuttons = ARRAY_SIZE(armadillo400_key_buttons),
};

static struct platform_device armadillo400_gpio_key_device = {
	.name ="gpio-keys",
	.id = 0,
};

extern void gpio_key_active(void);
static void __init armadillo400_init_key(void)
{
	gpio_key_active();
	mxc_register_device(&armadillo400_gpio_key_device,
			    &armadillo400_gpio_key_data);
}

static struct platform_pwm_backlight_data armadillo440_pwm_backlight_data = {
	.pwm_id = 0,
	.max_brightness = 255,
	.dft_brightness = 255,
	.pwm_period_ns = 10*1000*1000,
};

static struct platform_device armadillo400_pwm_backlight_device = {
	.name = "pwm-backlight",
};

extern void gpio_activate_pwm(int id);
static void armadillo400_pwm0_enable_pad(void)
{
	gpio_activate_pwm(0);
}

static struct mxc_pwm_platform_data armadillo400_pwm0_data = {
	.pwmo_invert = 1,
	.enable_pwm_pad = armadillo400_pwm0_enable_pad,
};

static struct platform_device mx25_mxcfb_device = {
	.name = "mxc_sdc_fb",
	.id = 0,
	.dev = {
		.coherent_dma_mask = 0xFFFFFFFF,
	},
};

static void __init armadillo440_init_fb(void)
{
#if defined(CONFIG_FB_MXC) || defined(CONFIG_FB_MXC_MODULE)
	mxc_register_device(&mx25_mxcfb_device, "FG040360DSSWBG03");

	mxc_register_device(&mxc_pwm_device0, &armadillo400_pwm0_data);
	mxc_register_device(&armadillo400_pwm_backlight_device,
			    &armadillo440_pwm_backlight_data);
#endif
}

static void armadillo400_fec_link_led_active(int active)
{
	gpio_direction_output(FEC_LINK_LED_GPIO, !active);
}

static void armadillo400_fec_enable(struct platform_device *pdev)
{
	/*
	 * Set up the GPIO pin.
	 *
	 * PHY_RST: gpio3[18] is ALT 5 mode of pin VSTBY_ACK
	 * LINK_LED: gpio3[16] is ALT 5 mode of pin UPLL_BYPCLK
	 */
	gpio_request(FEC_PHY_RST_GPIO, "PHY_RST");

	gpio_direction_output(FEC_PHY_RST_GPIO, 0);
	/* PHY spec says 100us min and Armadillo-400 has low pass filter
	 * for PHY_RST pin (300us). */
	udelay(1000);
	gpio_set_value(FEC_PHY_RST_GPIO, 1);

	gpio_request(FEC_LINK_LED_GPIO, "LINK_LED");
	armadillo400_fec_link_led_active(0);
}

static void armadillo400_fec_disable(struct platform_device *pdev)
{
	armadillo400_fec_link_led_active(0);
	gpio_direction_output(FEC_PHY_RST_GPIO, 0);

	gpio_free(FEC_PHY_RST_GPIO);
	gpio_free(FEC_LINK_LED_GPIO);
}

static int armadillo400_fec_suspend(struct platform_device *pdev,
				    pm_message_t state)
{
	armadillo400_fec_link_led_active(0);

	return 0;
}

static int armadillo400_fec_resume(struct platform_device *pdev)
{
	gpio_direction_output(FEC_PHY_RST_GPIO, 0);
	/* PHY spec says 100us min and Armadillo-400 has low pass filter
	 * for PHY_RST pin (300us). */
	udelay(1000);
	gpio_set_value(FEC_PHY_RST_GPIO, 1);

	return 0;
}

static struct fec_platform_data armadillo400_fec_data = {
	.enable			= armadillo400_fec_enable,
	.disable		= armadillo400_fec_disable,
	.suspend		= armadillo400_fec_suspend,
	.resume			= armadillo400_fec_resume,
	.link_led_active	= armadillo400_fec_link_led_active,
};

extern void gpio_fec_active(void);
static void __init armadillo400_init_fec(void)
{
	gpio_fec_active();
	mxc_register_device(&mx25_fec_device, &armadillo400_fec_data);
}

#if defined(CONFIG_ARMADILLO400_SDHC1_CON1)
static struct regulator_consumer_supply armadillo400_sdhc1_power_consumers[] = {
	REGULATOR_SUPPLY("VDD", "mxsdhci.0"),
};

static struct regulator_init_data armadillo400_sdhc1_power_init_data = {
	.constraints = {
		.valid_ops_mask = REGULATOR_CHANGE_STATUS,
	},
	.consumer_supplies = armadillo400_sdhc1_power_consumers,
	.num_consumer_supplies = ARRAY_SIZE(armadillo400_sdhc1_power_consumers),
};

static struct fixed_voltage_config armadillo400_sdhc1_power_config = {
	.supply_name = "sdhc1.pwrsw",
	.gpio = SDHC1_PWREN_GPIO,
	.enable_high = 0,
	.enabled_at_boot = 1,
	.startup_delay = 100000,
	.microvolts = 3300000,
	.init_data = &armadillo400_sdhc1_power_init_data,
};

static struct platform_device armadillo400_sdhc1_power = {
	.name = "reg-fixed-voltage",
	.id = 0,
};
#endif

#if defined(CONFIG_ARMADILLO400_SDHC2_PWREN_CON9_1)
static struct regulator_consumer_supply armadillo400_sdhc2_power_consumers[] = {
	REGULATOR_SUPPLY("VDD", "mxsdhci.1"),
};

static struct regulator_init_data armadillo400_sdhc2_power_init_data = {
	.constraints = {
		.valid_ops_mask = REGULATOR_CHANGE_STATUS,
	},
	.consumer_supplies = armadillo400_sdhc2_power_consumers,
	.num_consumer_supplies = ARRAY_SIZE(armadillo400_sdhc2_power_consumers),
};

static struct fixed_voltage_config armadillo400_sdhc2_power_config = {
	.supply_name = "sdhc2.pwrsw",
	.gpio = SDHC2_PWREN_GPIO,
	.enable_high = 0,
	.enabled_at_boot = 1,
	.startup_delay = 100000,
	.microvolts = 3300000,
	.init_data = &armadillo400_sdhc2_power_init_data,
};

static struct platform_device armadillo400_sdhc2_power = {
	.name = "reg-fixed-voltage",
	.id = 1,
};
#endif

static void __init armadillo400_init_regulator(void)
{
#if defined(CONFIG_ARMADILLO400_SDHC1_CON1)
	mxc_register_device(&armadillo400_sdhc1_power,
			    &armadillo400_sdhc1_power_config);
#endif
#if defined(CONFIG_ARMADILLO400_SDHC2_PWREN_CON9_1)
	mxc_register_device(&armadillo400_sdhc2_power,
			    &armadillo400_sdhc2_power_config);
#endif
}

/*
 * SDHC: Probe for the card. If present the GPIO data would be set.
 */
static unsigned int __maybe_unused sdhc_get_card_det_status(struct device *dev)
{
	if (to_platform_device(dev)->id == 0)
		return gpio_get_value(SDHC1_CD_GPIO);
	else
		return gpio_get_value(SDHC2_CD_GPIO);
}

#if defined(CONFIG_ARMADILLO400_SDHC1_CON1)
static struct mxc_mmc_platform_data armadillo400_sdhc1_data = {
	.ocr_mask = MMC_VDD_29_30 | MMC_VDD_32_33,
	.caps = MMC_CAP_4_BIT_DATA,
	.min_clk = 400000,
	.max_clk = 52000000,
	.card_inserted_state = 0,
	.status = sdhc_get_card_det_status,
	.clock_mmc = "esdhc_clk",
	.power_mmc = "sdhc1.pwrsw",
};

static struct resource mxcsdhc1_resources[] = {
	[0] = {
		.start = MX25_MMC_SDHC1_BASE_ADDR,
		.end = MX25_MMC_SDHC1_BASE_ADDR + SZ_4K - 1,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = MX25_INT_SDHC1,
		.end = MX25_INT_SDHC1,
		.flags = IORESOURCE_IRQ,
	},
	[2] = {
		.start = IOMUX_TO_IRQ(MX25_PIN_NFRB),
		.end = IOMUX_TO_IRQ(MX25_PIN_NFRB),
		.flags = IORESOURCE_IRQ,
	},
};

static struct platform_device mxcsdhc1_device = {
	.name = "mxsdhci",
	.id = 0,
	.dev = {
		.platform_data = &armadillo400_sdhc1_data,
	},
	.num_resources = ARRAY_SIZE(mxcsdhc1_resources),
	.resource = mxcsdhc1_resources,
};
#endif

#if defined(CONFIG_ARMADILLO400_SDHC2_CON9)
static struct mxc_mmc_platform_data armadillo400_sdhc2_data = {
	.ocr_mask = MMC_VDD_29_30 | MMC_VDD_32_33,
	.caps = MMC_CAP_4_BIT_DATA,
	.min_clk = 400000,
	.max_clk = 52000000,
	.card_inserted_state = 0,
	.status = sdhc_get_card_det_status,
	.clock_mmc = "esdhc_clk",
#if defined(CONFIG_ARMADILLO400_SDHC2_PWREN_CON9_1)
	.power_mmc = "sdhc2.pwrsw",
#endif
};

static struct resource mxcsdhc2_resources[] = {
	[0] = {
		.start = MX25_MMC_SDHC2_BASE_ADDR,
		.end = MX25_MMC_SDHC2_BASE_ADDR + SZ_4K - 1,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = MX25_INT_SDHC2,
		.end = MX25_INT_SDHC2,
		.flags = IORESOURCE_IRQ,
	},
	[2] = {
		.start = IOMUX_TO_IRQ(MX25_PIN_CSI_D9),
		.end = IOMUX_TO_IRQ(MX25_PIN_CSI_D9),
		.flags = IORESOURCE_IRQ,
	},
};

static struct platform_device mxcsdhc2_device = {
	.name = "mxsdhci",
	.id = 1,
	.dev = {
		.platform_data = &armadillo400_sdhc2_data,
	},
	.num_resources = ARRAY_SIZE(mxcsdhc2_resources),
	.resource = mxcsdhc2_resources,
};
#endif

extern void gpio_sdhc_active(int port);
extern void gpio_sdhc_inactive(int port);
static void __init armadillo400_init_sdhc(void)
{
#if defined(CONFIG_ARMADILLO400_SDHC1_CON1)
	platform_device_register(&mxcsdhc1_device);
#endif
#if defined(CONFIG_ARMADILLO400_SDHC2_CON9)
	platform_device_register(&mxcsdhc2_device);
#endif
}

extern void gpio_i2c_active(int i2c_num);
static inline int __armadillo400_i2c_private_init(int port)
{
	gpio_i2c_active(port);
	return 0;
}

static int armadillo400_i2c1_private_init(struct device *dev)
{
	return __armadillo400_i2c_private_init(0);
}

static struct imxi2c_platform_data armadillo400_i2c1_data = {
	.bitrate = 40000,
	.init = armadillo400_i2c1_private_init,
};

static struct i2c_board_info armadillo400_i2c1_board_info[] __initdata = {
	{
		.type = "mc34704",
		.addr = 0x54,
	},
};

#if defined(CONFIG_ARMADILLO400_I2C2_CON14)
static int armadillo400_i2c2_private_init(struct device *dev)
{
	return __armadillo400_i2c_private_init(1);
}

static struct imxi2c_platform_data armadillo400_i2c2_data = {
	.bitrate = 40000,
	.init = armadillo400_i2c2_private_init,
};

static struct i2c_board_info armadillo400_i2c2_board_info[] __initdata = {
	{
		.type = "s35390a",
		.addr = 0x30,
	},
};
#endif

#if defined(CONFIG_ARMADILLO400_I2C3_CON11)
static int armadillo400_i2c3_private_init(struct device *dev)
{
	return __armadillo400_i2c_private_init(2);
}

static struct imxi2c_platform_data armadillo400_i2c3_data = {
	.bitrate = 40000,
	.init = armadillo400_i2c3_private_init,
};

static struct i2c_board_info armadillo400_i2c3_board_info[] __initdata = {
	{
		.type = "s35390a",
		.addr = 0x30,
	},
	{
		.type = "wm8978",
		.addr = 0x1a,
	},
};
#endif

static void __init armadillo400_init_i2c(void)
{
	mxc_register_device(&mxc_i2c_device0,
			    &armadillo400_i2c1_data);
	i2c_register_board_info(0, armadillo400_i2c1_board_info,
				ARRAY_SIZE(armadillo400_i2c1_board_info));

#if defined(CONFIG_ARMADILLO400_I2C2_CON14)
	mxc_register_device(&mxc_i2c_device1,
			    &armadillo400_i2c2_data);
	i2c_register_board_info(1, armadillo400_i2c2_board_info,
				ARRAY_SIZE(armadillo400_i2c2_board_info));
#endif

#if defined(CONFIG_ARMADILLO400_I2C3_CON11)
	mxc_register_device(&mxc_i2c_device2,
			    &armadillo400_i2c3_data);
	i2c_register_board_info(2, armadillo400_i2c3_board_info,
				ARRAY_SIZE(armadillo400_i2c3_board_info));
#endif
}

#define MTD_PART(_name, _size, _offset, _mask_flags) \
	{                                            \
		.name		= _name,             \
		.size		= _size,             \
		.offset		= _offset,           \
		.mask_flags	= _mask_flags,       \
	}

static struct mtd_partition armadillo400_mtd_nor_partitions_8MB[] = {
	MTD_PART("nor.bootloader",   4* 32*1024, 0,
		 MTD_WRITEABLE /* force read-only */ ),
	MTD_PART("nor.kernel",      16*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.userland",    46*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.config",       1*128*1024, MTDPART_OFS_APPEND, 0),
};

static struct mtd_partition armadillo400_mtd_nor_partitions_16MB[] = {
	MTD_PART("nor.bootloader",   4* 32*1024, 0,
		 MTD_WRITEABLE /* force read-only */ ),
	MTD_PART("nor.kernel",      16*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.userland",   110*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.config",       1*128*1024, MTDPART_OFS_APPEND, 0),
};

static struct mtd_partition armadillo400_mtd_nor_partitions_32MB[] = {
	MTD_PART("nor.bootloader",   4* 32*1024, 0,
		 MTD_WRITEABLE /* force read-only */ ),
	MTD_PART("nor.kernel",      16*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.userland",   238*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.config",       1*128*1024, MTDPART_OFS_APPEND, 0),
};

static struct mtd_partition armadillo400_mtd_nor_partitions_64MB[] = {
	MTD_PART("nor.bootloader",   4* 32*1024, 0,
		 MTD_WRITEABLE /* force read-only */ ),
	MTD_PART("nor.kernel",      16*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.userland",   494*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.config",       1*128*1024, MTDPART_OFS_APPEND, 0),
};

static int armadillo400_mtd_nor_update_partitions(
	struct map_info *map,
	struct flash_platform_data *plat)
{
	struct cfi_private *cfi = map->fldrv_priv;

	switch (cfi->cfiq->DevSize) {
	case 23: /* 8MB */
		plat->parts	= armadillo400_mtd_nor_partitions_8MB;
		plat->nr_parts	= ARRAY_SIZE(armadillo400_mtd_nor_partitions_8MB);
		break;
	case 24: /* 16MB */
		break;
	case 25: /* 32MB */
		plat->parts	= armadillo400_mtd_nor_partitions_32MB;
		plat->nr_parts	= ARRAY_SIZE(armadillo400_mtd_nor_partitions_32MB);
		break;
	case 26: /* 64MB */
		plat->parts	= armadillo400_mtd_nor_partitions_64MB;
		plat->nr_parts	= ARRAY_SIZE(armadillo400_mtd_nor_partitions_64MB);
		break;
	default:
		printk("Not support flash-size.\n");
		return -1;
	}

	return 0;
}

static struct armadillo_flash_private_data armadillo400_mtd_nor_data = {
	.plat = {
		.map_name	= "cfi_probe",
		.width		= 2,
		.parts		= armadillo400_mtd_nor_partitions_16MB,
		.nr_parts	= ARRAY_SIZE(armadillo400_mtd_nor_partitions_16MB),
	},
	.update_partitions = armadillo400_mtd_nor_update_partitions,
	.map_name = "armadillo-nor",
};

static struct resource armadillo400_mtd_nor_resource = {
	.start		= MX25_CS0_BASE_ADDR,
	.end		= MX25_CS0_BASE_ADDR + MX25_CS0_SIZE - 1,
	.flags		= IORESOURCE_MEM,
};

struct platform_device armadillo400_mtd_nor_device = {
	.name		= "armadillo-nor",
	.id		= 0,
	.num_resources	= 1,
	.resource	= &armadillo400_mtd_nor_resource,
};

static struct mx25_wdt_platform_data armadillo400_wdt_pdata = {
	.interval_ms	= 1000,
};

static struct platform_device mx25_wdt_device = {
	.name		= "mx25-wdt",
};

static struct imxuart_platform_data armadillo400_uart2_pdata = {
	.flags = IMXUART_HAVE_RTSCTS,
};

static void __init armadillo400_init(void)
{
	mxc_register_device(&mx25_wdt_device, &armadillo400_wdt_pdata);
	mxc_register_device(&mxc_uart_device2, &armadillo400_uart2_pdata);
	mxc_register_device(&armadillo400_mtd_nor_device,
			    &armadillo400_mtd_nor_data);

	armadillo400_init_regulator();
	armadillo400_init_i2c();
	armadillo400_init_sdhc();
	armadillo400_init_fec();
	armadillo400_init_key();
	armadillo400_init_led();
	armadillo400_init_gpio();
}

static void __init armadillo420_init(void)
{
	armadillo400_init();
}

static void __init armadillo440_init(void)
{
	armadillo400_init();

	armadillo440_init_fb();
	armadillo440_init_audio();
}

static void __init armadillo400_timer_init(void)
{
	mx25_clocks_init();
}

static struct sys_timer armadillo400_timer = {
	.init   = armadillo400_timer_init,
};

#if defined(CONFIG_MACH_ARMADILLO420)
MACHINE_START(ARMADILLO420, "Armadillo-420")
	/* Maintainer: Atmark Techno, Inc. */
	.phys_io	= MX25_AIPS1_BASE_ADDR,
	.io_pg_offst	= ((MX25_AIPS1_BASE_ADDR_VIRT) >> 18) & 0xfffc,
	.boot_params	= PHYS_OFFSET + 0x100,
	.map_io		= mx25_map_io,
	.init_irq	= mx25_init_irq,
	.init_machine	= armadillo420_init,
	.timer		= &armadillo400_timer,
MACHINE_END
#endif

#if defined(CONFIG_MACH_ARMADILLO440)
MACHINE_START(ARMADILLO440, "Armadillo-440")
	/* Maintainer: Atmark Techno, Inc. */
	.phys_io	= MX25_AIPS1_BASE_ADDR,
	.io_pg_offst	= ((MX25_AIPS1_BASE_ADDR_VIRT) >> 18) & 0xfffc,
	.boot_params	= PHYS_OFFSET + 0x100,
	.map_io		= mx25_map_io,
	.init_irq	= mx25_init_irq,
	.init_machine	= armadillo440_init,
	.timer		= &armadillo400_timer,
MACHINE_END
#endif
